/*****************************************************************/ 
/**		     Microsoft LAN Manager			**/ 
/**	       Copyright(c) Microsoft Corp., 1987-1990		**/ 
/*****************************************************************/ 

/********************************************************************
 *								    *
 *  About this file ...  NMPIPE.H				    *
 *								    *
 *  This file contains information about the Named Pipe APIs.	    *
 *								    *
 *	Function prototypes.					    *
 *								    *
 *	Data structure templates.				    *
 *								    *
 *	Definition of special values.				    *
 *								    *
 ********************************************************************/

/****************************************************************
 *                                                              *
 *              Type definition                                 *
 *                                                              *
 ****************************************************************/

typedef void (far pascal *API_CALLBACK)(unsigned long);

/**************************************************************** 
 *								*
 *	  	Function prototypes 				*
 *								*
 ****************************************************************/

extern API_FUNCTION
  DosReadAsyncNmPipe(unsigned short, API_CALLBACK, unsigned short far *,
                                unsigned char far *, unsigned short,
                                unsigned short far *);

extern API_FUNCTION
  DosWriteAsyncNmPipe(unsigned short, API_CALLBACK, unsigned short far *,
                                unsigned char far *, unsigned short,
                                unsigned short far *);

extern API_FUNCTION
  DosReadAsyncNmPipe2(unsigned short, API_CALLBACK, unsigned short far *,
                                unsigned char far *, unsigned short,
                                unsigned short far *, unsigned long);

extern API_FUNCTION
  DosWriteAsyncNmPipe2(unsigned short, API_CALLBACK, unsigned short far *,
                                unsigned char far *, unsigned short,
                                unsigned short far *, unsigned long);

/* The following functions and constants are already defined in OS2.H
 * if INCL_DOSNMPIPES flag has been defined.  Therefore, we don't want
 * need them here.
 */

#ifndef INCL_DOSNMPIPES

extern API_FUNCTION
  DosMakeNmPipe(char FAR *, unsigned FAR *, unsigned short, unsigned short,
				unsigned short, unsigned short, long);

extern API_FUNCTION
  DosQNmPipeInfo(unsigned, unsigned short, char FAR *, unsigned short);

extern API_FUNCTION
  DosConnectNmPipe(unsigned);

extern API_FUNCTION
  DosDisconnectNmPipe(unsigned);

extern API_FUNCTION
  DosQNmPHandState(unsigned, unsigned short FAR *);

extern API_FUNCTION
  DosSetNmPHandState(unsigned, unsigned short);

extern API_FUNCTION
  DosPeekNmPipe(unsigned, char FAR *, unsigned short, unsigned short FAR *,
				unsigned short FAR *, unsigned short FAR *);

extern API_FUNCTION
  DosWaitNmPipe(char FAR *, long);

extern API_FUNCTION
  DosTransactNmPipe(unsigned, char FAR *, unsigned short, char FAR *,
				unsigned short, unsigned short FAR *);

extern API_FUNCTION
  DosCallNmPipe(char FAR *, char FAR *, unsigned short, char FAR *,
				unsigned short, unsigned short FAR *, long);

extern API_FUNCTION
  DosSetNmPipeSem(unsigned, long, unsigned short);

extern API_FUNCTION
  DosQNmPipeSemState(long, char FAR *, unsigned short);

/**INTERNAL_ONLY**/
extern API_FUNCTION
  DosRawWriteNmPipe(unsigned, char FAR *, unsigned short, unsigned short FAR *);

extern API_FUNCTION
  DosRawReadNmPipe(unsigned, char FAR *, unsigned short, unsigned short FAR *);
/**END_INTERNAL**/

/**************************************************************** 
 *								*
 *	  	Data structure templates			*
 *								*
 ****************************************************************/


struct	npi_data1 {	/* PipeInfo data block (returned, level 1) */
	unsigned short	npi_obuflen;	/* length of outgoing I/O buffer */
	unsigned short	npi_ibuflen;	/* length of incoming I/O buffer */
	unsigned char	npi_maxicnt;	/* maximum number of instances */
	unsigned char	npi_curicnt;	/* current number of instances */
	unsigned char	npi_namlen;	/* length of pipe name */
	char	npi_name[1];		/* start of name */
};	/* npi_data1 */

struct	npss	{	/* QNmPipeSemState information record */
	unsigned char	npss_status;	/* type of record, 0 = EOI, 1 = read ok,
					 *   2 = write ok, 3 = pipe closed */
	unsigned char	npss_flag;	/* additional info, 01 = waiting thread */
	unsigned short	npss_key;	/* user's key value */
	unsigned short	npss_avail;	/* available data/space if status = 1/2 */
};	/* npss */

/* values in npss_status */
#define	NPSS_EOI	0	/* End Of Information */
#define	NPSS_RDATA	1	/* read data available */
#define	NPSS_WSPACE	2	/* write space available */
#define	NPSS_CLOSE	3	/* pipe in CLOSING state */

/* values in npss_flag */
#define	NPSS_WAIT	0x01	/* waiting thread on other end of pipe */


/**************************************************************** 
 *								*
 *	  	Special values and constants			*
 *								*
 ****************************************************************/

/* defined bits in pipe mode */
#define	NP_NBLK		0x8000			/* non-blocking read/write */
#define	NP_SERVER	0x4000			/* set if server end */
#define	NP_WMESG	0x0400			/* write messages */
#define	NP_RMESG	0x0100			/* read as messages */
#define	NP_ICOUNT	0x00FF			/* instance count field */

/*NOINC*/
#define NP_INDEFINITE_WAIT	(-1L)
#define NP_DEFAULT_WAIT		0L
/*INC*/

/* Pipe states */

#define NP_STATE_DISCONNECTED	0x0001
#define NP_STATE_LISTENING	0x0002
#define NP_STATE_CONNECTED	0x0003
#define NP_STATE_CLOSING	0x0004

/* Pipe open modes */

#define NP_ACCESS_INBOUND	0x0000
#define NP_ACCESS_OUTBOUND	0x0001
#define NP_ACCESS_DUPLEX	0x0002
#define NP_INHERIT		0x0000
#define NP_NOINHERIT		0x0080
#define NP_WRITEBEHIND		0x0000
#define NP_NOWRITEBEHIND	0x4000

/* QueryNPHState state */

#define NP_READMODE_BYTE	0x0000
#define NP_READMODE_MESSAGE	0x0100
#define NP_TYPE_BYTE		0x0000
#define NP_TYPE_MESSAGE		0x0400
#define NP_END_CLIENT		0x0000
#define NP_END_SERVER		0x4000
#define NP_WAIT			0x0000
#define NP_NOWAIT		0x8000
#define NP_UNLIMITED_INSTANCES	0x00FF


/*	Named pipes may be in one of several states depending on the actions
 *	that have been taken on it by the server end and client end.  The
 *	following state/action table summarizes the valid state transitions:
 *
 *	Current state		Action			Next state
 *
 *	 <none>		    server DosMakeNmPipe	DISCONNECTED
 *	 DISCONNECTED	    server connect		LISTENING
 *	 LISTENING	    client open			CONNECTED
 *	 CONNECTED	    server disconn		DISCONNECTED
 *	 CONNECTED	    client close		CLOSING
 *	 CLOSING	    server disconn		DISCONNECTED
 *	 CONNECTED	    server close		CLOSING
 *	 <any other>	    server close		<pipe deallocated>
 *
 *	If a server disconnects his end of the pipe, the client end will enter a
 *	special state in which any future operations (except close) on the file
 *	descriptor associated with the pipe will return an error.
 */

/*
 *	Values for named pipe state
 */

#define	NP_DISCONNECTED	1		/* after pipe creation or Disconnect */
#define	NP_LISTENING	2		/* after DosNmPipeConnect */
#define	NP_CONNECTED	3		/* after Client open */
#define	NP_CLOSING	4		/* after Client or Server close */

#endif  /* INCL_DOSNMPIPES */
