/****************************************************************************/
/*                                                                           */
/*   Module Name: GCCI_CM.H                                                  */
/*                                                                           */
/*   Description: GCCI Symbol and Structure definitions                      */
/*                                                                           */
/*   Property of IBM.                                                        */
/*   This sample code is available without warranty upon request.            */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                   PCM_CALL_SETUP Request Primitive                        */
/*****************************************************************************/

/* Control Information for PCM_CALL_SETUP Request */
typedef struct CI_PCM_Call_Setup_Req
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_SETUP_REQ)     */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_SETUP_REQ     */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_P_SAP_ID         */
      DWORD             P_SAP_ID;       /* Provide assigned SAP id           */
      DWORD             U_CI_ID;        /* User Assigned Call Instance ID    */
   } CI_PCM_CALL_SETUP_REQ;

/* Primitive Code for PCM_CALL_SETUP Request */
#define GCCI_CM_PC_PCM_CALL_SETUP_REQ   0x0C10


/*****************************************************************************/
/*                   PCM_CALL_SETUP Confirm Primitive                        */
/*****************************************************************************/

/* Control Information for PCM_CALL_SETUP Confirm */
typedef struct CI_PCM_Call_Setup_Cnf
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_SETUP_CNF)     */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_SETUP_CNF     */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_CI_ID          */
      DWORD             U_CI_ID;        /* User assigned Call Instance Id    */
      GCCI_COM_COMSTATUS CommonStatus;  /*                                   */
      DWORD             P_CI_ID;        /* Provider assigned Call Instance Id*/
   } CI_PCM_CALL_SETUP_CNF;

/* Primitive Code for PCM_CALL_SETUP Confirm */
#define GCCI_CM_PC_PCM_CALL_SETUP_CNF   0xCC10


/*****************************************************************************/
/*                  PCM_CALL_SETUP Indication Primitive                      */
/*****************************************************************************/

/* Control Information for PCM_CALL_SETUP Indication */
typedef struct CI_PCM_Call_Setup_Ind
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_SETUP_IND)     */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_SETUP_IND     */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_SAP_ID         */
      DWORD             U_SAP_ID;       /* User assigned SAP Id              */
      DWORD             P_CI_ID;        /* Provider assigned Call Instance Id*/
   } CI_PCM_CALL_SETUP_IND;

/* Primitive Code for PCM_CALL_SETUP Indication */
#define GCCI_CM_PC_PCM_CALL_SETUP_IND   0x4C10


/*****************************************************************************/
/*                   PCM_CALL_SETUP Response Primitive                       */
/*****************************************************************************/

/* Control Information for PCM_CALL_SETUP Response */
typedef struct CI_PCM_Call_Setup_Rsp
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_SETUP_RSP)     */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_SETUP_RSP     */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_P_CI_ID          */
      DWORD             P_CI_ID;        /* Provider assigned Call Instance Id*/
      DWORD             U_CI_ID;        /* User assigned Call Instance Id    */
      WORD              ActionCode;     /*                                   */
   } CI_PCM_CALL_SETUP_RSP;

/* Primitive Code for PCM_CALL_SETUP Response */
#define GCCI_CM_PC_PCM_CALL_SETUP_RSP   0x8C10

/* Action Code */
#define GCCI_CM_ACT_CODE_ACCEPT_CALL    0x0000
#define GCCI_CM_ACT_CODE_REJECT_CALL    0x0001
#define GCCI_CM_ACT_CODE_PENDING        0x0002


/*****************************************************************************/
/*                   PCM_CALL_CLEAR Request Primitive                        */
/*****************************************************************************/


/* Control Information for PCM_CALL_CLEAR Request */
typedef struct CI_PCM_Call_Clear_Req
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_CLEAR_REQ)     */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_CLEAR_REQ     */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_P_CI_ID OR       */
                                        /* GCCI_COM_ID_TYPE_P_SAP_ID         */
      DWORD             Identifier;     /*                                   */
      DWORD             U_CI_ID;        /* User assigned Call Instance Id    */
   } CI_PCM_CALL_CLEAR_REQ;


/* Primitive Code for PCM_CALL_CLEAR Request */
#define GCCI_CM_PC_PCM_CALL_CLEAR_REQ   0x0C11

/* Notes: CM  may submit a Call Clear request before receiving               */
/*  a Confirm to a prior Call Setup Request. Since in this case              */
/*  CM  does not yet have a P_CI_ID, it will set the IDType parameter        */
/*  to CMI_CM_ID_TYPE_P_SAP_ID, and put the P_SAP_ID into the Identifier     */
/*  parameter. In this case CM will also set the U_CI_ID                     */
/*  parameter  to the Call Instance being cleared.                           */


/*****************************************************************************/
/*                   PCM_CALL_CLEAR Confirm Primitive                        */
/*****************************************************************************/

/* Control Information for PCM_CALL_CLEAR Confirm */
typedef struct CI_PCM_Call_Clear_Cnf
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_CLEAR_CNF)     */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_CLEAR_CNF     */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_CI_ID          */
      DWORD             U_CI_ID;        /* User assigned Call Instance Id    */
      GCCI_COM_COMSTATUS CommonStatus;  /*                                   */
   } CI_PCM_CALL_CLEAR_CNF;

/* Primitive Code for PCM_CALL_CLEAR Confirm */
#define GCCI_CM_PC_PCM_CALL_CLEAR_CNF   0xCC11


/*****************************************************************************/
/*                  PCM_CALL_CLEAR Indication Primitive                      */
/*****************************************************************************/

/* Control Information for PCM_CALL_CLEAR Indication */
typedef struct CI_PCM_Call_Clear_Ind
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_CLEAR_IND)     */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_CLEAR_IND     */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_CI_ID OR       */
                                        /* GCCI_COM_ID_TYPE_U_SAP_ID         */
      DWORD             Identifier;     /*                                   */
      GCCI_COM_COMSTATUS CommonStatus;  /*                                   */
      DWORD             P_CI_ID;        /* Provider assigned Call Instance Id*/
   } CI_PCM_CALL_CLEAR_IND;

/* Primitive Code for PCM_CALL_CLEAR Indication */
#define GCCI_CM_PC_PCM_CALL_CLEAR_IND   0x4C11

/* Notes: The PCM may issue a Call Clear Indicate before it has received a   */
/*  Call Setup response to a prior Call Setup indicate. Since in this case   */
/*  the PCM does not yet have a U_CI_ID, it will set the IDType parameter to */
/*  CMI_CM_ID_U_SAP_ID, and put the U_SAP_ID into the Identifier field.      */
/*  In this case PCM will also set the P_CI_ID parameter to the provider     */
/*  Call Instance being cleared.                                             */


/*****************************************************************************/
/*                  PCM_CALL_CLEAR Response Primitive                        */
/*****************************************************************************/

/* Control Information for PCM_CALL_CLEAR Response */
typedef struct CI_PCM_Call_Clear_Rsp
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_CLEAR_RSP)     */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_CLEAR_RSP     */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_P_CI_ID          */
      DWORD             P_CI_ID;        /* Provider assigned Call Instance Id*/
   } CI_PCM_CALL_CLEAR_RSP;

/* Primitive Code for PCM_CALL_CLEAR Response */
#define GCCI_CM_PC_PCM_CALL_CLEAR_RSP   0x8C11


/*****************************************************************************/
/*                PCM_CALL_CONNECTED Indication Primitive                    */
/*****************************************************************************/

/* Control Information for PCM_CALL_CONNECTED Indication */
typedef struct CI_PCM_Call_Cntd_Ind
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_CNTD_IND)      */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_CNTD_IND      */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_CI_ID          */
      DWORD             U_CI_ID;        /* User assigned Call Instance Id    */
   } CI_PCM_CALL_CNTD_IND;

/* Primitive Code for PCM_CALL_CONNECTED Indication */
#define GCCI_CM_PC_PCM_CALL_CNTD_IND    0x4C12


/*****************************************************************************/
/*                   PCM_CALL_CONTROL Request Primitive                        */
/*****************************************************************************/

/* Control Information for PCM_CALL_CONTROL Request */
typedef struct CI_PCM_Call_Ctrl_Req
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_CTRL_REQ)      */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_CTRL_REQ      */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_P_CI_ID          */
      DWORD             P_CI_ID;        /* Provider assigned call instance Id*/
   } CI_PCM_CALL_CTRL_REQ;

/* Primitive Code for PCM_CALL_CONTROL Request */
#define GCCI_CM_PC_PCM_CALL_CTRL_REQ    0x0C1A


/*****************************************************************************/
/*                   PCM_CALL_CONTROL Confirm Primitive                        */
/*****************************************************************************/

/* Control Information for PCM_CALL_CONTROL Confirm */
typedef struct CI_PCM_Call_Ctrl_Cnf
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_CTRL_CNF)      */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_CTRL_CNF)     */
      WORD              DataInfoLength; /* 0                                 */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_CI_ID          */
      DWORD             U_CI_ID;        /* User assigned call instance Id    */
      GCCI_COM_COMSTATUS CommonStatus;  /*                                   */
   } CI_PCM_CALL_CTRL_CNF;

/* Primitive Code for PCM_CALL_CONTROL Confirm */
#define GCCI_CM_PC_PCM_CALL_CTRL_CNF    0xCC1A


/*****************************************************************************/
/*                  PCM_CALL_STATUS Indication Primitive                     */
/*****************************************************************************/

/* Control Information for PCM_CALL_STATUS Indication */
typedef struct CI_PCM_Call_Stat_Ind
   {
      WORD              CtrlInfoLength; /* sizeof(CI_PCM_CALL_STAT_IND)      */
      WORD              PrimitiveCode;  /* GCCI_CM_PC_PCM_CALL_STAT_IND      */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_CI_ID          */
      DWORD             U_CI_ID;        /* User assigned Call Instance Id    */
      GCCI_COM_COMSTATUS CommonStatus;  /*                              @A1A */
   } CI_PCM_CALL_STAT_IND;

/* Primitive Code for PCM_CALL_STATUS Indication */
#define GCCI_CM_PC_PCM_CALL_STAT_IND    0x4C14

