/*****************************************************************************/
/*                                                                           */
/*   Module      : GCCI_COM.H                                                */
/*                                                                           */
/*   Description : GCCI Symbol and Structure definitions                     */
/*                                                                           */
/*   Property of IBM.                                                        */
/*   This sample code is available without warranty upon request.            */
/*                                                                           */
/*****************************************************************************/

#define GCCI_COM 1

#include "wcl1632.h"

/*****************************************************************************/
/*                   Common Definitions                                      */
/*****************************************************************************/
#define GCCI_COM_SFID_PRM_SPECIFIC      0x04

/*****************************************************************************/
/*                                                                           */
/*                            Common Status                                  */
/*                                                                           */
/*****************************************************************************/

typedef struct GCCIComStatus
   {
      BYTE              CompCodeCategory;
      BYTE              ReportLocation;
      WORD              ReportCompCode;
      BYTE              Reserved;
      BYTE              OrigLocation;
      WORD              OrigCompCode;
      DWORD             SysLogID;
   } GCCI_COM_COMSTATUS;


/* CompCodeCatagory */
#define GCCI_COM_CMP_CAT_REQ_SUCCESSFUL 0x00
#define GCCI_COM_CMP_CAT_REQ_REJECTED   0x08
#define GCCI_COM_CMP_CAT_REQ_ERROR      0x10
#define GCCI_COM_CMP_CAT_STATE_ERROR    0x20
#define GCCI_COM_CMP_CAT_USAGE_ERROR    0x40
#define GCCI_COM_CMP_CAT_PERM_ERROR     0x80


/* ReportLocation/OrigLocation */
#define GCCI_COM_LOC_LOCAL              0x10
#define GCCI_COM_LOC_PATH               0x20
#define GCCI_COM_LOC_LINE               0x20
#define GCCI_COM_LOC_REMOTE             0x30

/* Layer ID Code */
#define GCCI_COM_LAYER_ID_CM            0x00
#define GCCI_COM_LAYER_ID_PCM           0x02
#define GCCI_COM_LAYER_ID_DLC           0x06
#define GCCI_COM_LAYER_ID_MAC           0x0A
/* Note: Reporting/Origination Location and Layer ID Code are or'ed into a */
/*       single byte (ReportLocation or OrigLocation)                      */


/*****************************************************************************/
/*                                                                           */
/*                  PCM_ACTIVATE_SAP Request Primitive                       */
/*                                                                           */
/*****************************************************************************/

/* Control Information for PCM_ACTIVATE_SAP Request */
typedef struct GCCIComCIActSAPReq
   {
      WORD              CtrlInfoLength; /* sizeof(GCCI_COM_CI_ACT_SAP_REQ)   */
      WORD              PrimitiveCode;  /* GCCI_COM_PC_ACT_SAP_REQ           */
      WORD              DataInfoLength; /* sizeof(GCCI_COM_SF01_ACT_SAP_REQ) */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_NONE             */
      DWORD             Identifier;     /* 0                                 */
      DWORD             U_SAP_ID;       /* User assigned SAP id              */
      BYTE              Reserved1;      /* 0                                 */
      BYTE              ServiceType;    /*                                   */
      BYTE              PCMName[16];    /* ASCIIZ PCM Name from Module Enbler*/
   } GCCI_COM_CI_ACT_SAP_REQ;

/* PrimitiveCode */
#define GCCI_COM_PC_ACT_SAP_REQ         0x0C0D

/* ServiceType */
#define GCCI_COM_SRV_TYPE_ERROR_LOG_MGR 0x04
#define GCCI_COM_SRV_TYPE_CNM_MGR       0x08
#define GCCI_COM_SRV_TYPE_CONNECT_MGR   0x10

/* Primitive Specific Subfield for PCM_ACTIVATE_SAP Request */
typedef struct GCCIComSF01ActSAPReq
   {
      WORD              SFLength;       /* sizeof(GCCI_COM_SF01_ACT_SAP_REQ)*/
      BYTE              SFIdentifier;   /* GCCI_COM_SFID_PRM_SPECIFIC       */
      BYTE              SFType;         /* GCCI_COM_SF_LOWER_DISPATCH       */
      WORD (CALL16PTR   SFCMIndEP)(VOID PTR16); /* CM Indication Entry Pt address   */
      WORD (CALL16PTR   SFCMCnfEP)(VOID PTR16); /* CM Confirm Entry Point address   */
      WORD (CALL16PTR   SFCNMIndEP)(VOID PTR16);/* CNM Indication Entry Pt address  */
      WORD (CALL16PTR   SFCNMCnfEP)(VOID PTR16);/* CNM Confirm Entry Point address  */
   } GCCI_COM_SF01_ACT_SAP_REQ;

/* Subfield Type       */
#define GCCI_COM_SF_LOWER_DISPATCH      0x01

/*****************************************************************************/
/*                                                                           */
/*                  PCM_ACTIVATE_SAP Confirm Primitive                       */
/*                                                                           */
/*****************************************************************************/

/* Control Information for PCM_ACTIVATE_SAP Confirm */
typedef struct GCCIComCIActSAPCnf
   {
      WORD              CtrlInfoLength; /* sizeof(GCCI_COM_CI_ACT_SAP_CNF)   */
      WORD              PrimitiveCode;  /* GCCI_COM_PC_ACT_SAP_CNF           */
      WORD              DataInfoLength; /* length of Data Information Area   */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_SAP_ID         */
      DWORD             U_SAP_ID;       /* User assigned SAP id              */
      GCCI_COM_COMSTATUS CommonStatus;  /*                                   */
      DWORD             P_SAP_ID;       /* Provider assigned SAP id          */
      BYTE              Reserved1[16];  /* 0                                 */
   } GCCI_COM_CI_ACT_SAP_CNF;

/* PrimitiveCode */
#define GCCI_COM_PC_ACT_SAP_CNF         0xCC0D


/* Primitive Specific Subfield #1 for PCM_ACTIVATE_SAP Confirm */
typedef struct LocalNumInfo
   {
      BYTE              NumberType;     /*                                   */
      BYTE              NumberPlan;     /*                                   */
      BYTE              NumberLength;   /* Number of digits                  */
      BYTE              NumberDigits[1];/* IA5 digits                        */
   } LOCALNUMINFO;
/* Number Type */
#define SF_NUMBER_TYPE_UNSPECIFIED      0x00
#define SF_NUMBER_TYPE_INTERNATIONAL    0x01
#define SF_NUMBER_TYPE_NATIONAL         0x02
#define SF_NUMBER_TYPE_NETWORK_SPECIFIC 0x03
#define SF_NUMBER_TYPE_SUBSCRIBER       0x04
#define SF_NUMBER_TYPE_ABBREVIATED      0x06

/* Numbering Plan */
#define SF_NUMBER_PLAN_UNKNOWN          0x00
#define SF_NUMBER_PLAN_ISDN             0x01
#define SF_NUMBER_PLAN_PRIVATE          0x09


typedef struct ChannelInfo                                       
   {
      DWORD             BMacAddr;
      BYTE              PermConnName[32];                        
   } CHANNELINFO;

typedef struct GCCIComSF01ActSAPCnf
   {
      WORD              SFLength;       /* Variable                          */
      BYTE              SFIdentifier;   /* GCCI_COM_SFID_PRM_SPECIFIC        */
      BYTE              SFType;         /* GCCI_COM_SF_PCM_INFORMATION       */
      BYTE              NetworkType;    /* GCCI_COM_NET_TYPE_ISDN            */
      BYTE              LocalNumCount;  /* Number of local Numbers           */
      WORD              BChannelsDef;   /* Number of defined B channels      */
      CHANNELINFO       ChanInfoArr[1]; /* Array of channel info.s           */
      LOCALNUMINFO      LocalNumArr[1]; /* Array of Local Numbers            */
   } GCCI_COM_SF01_ACT_SAP_CNF;

/* Subfield Type       */
#define GCCI_COM_SF_PCM_INFORMATION     0x01

/* NetworkType */
#define GCCI_COM_NET_TYPE_ISDN          0x01

                                        /* support for VSERIES PCM          */
typedef struct VChannelInfo                                                
   {                                                                       
      DWORD             VMacAddr;       /* VSERIES MAC address              */
      BYTE              VPermName[32];  /* Permanent Connect Name           */
      BYTE              VSync;          /* Channel Synchronicity            */
      BYTE              VRsvd;          /*                                  */
      WORD              VModem;         /* modem Class                      */
   } VCHANNELINFO;

/* Channel Synchronicity */
#define CHANNEL_UNSPECIFIED             0x00
#define CHANNEL_ASYNCHRONOUS            0x01
#define CHANNEL_SYNCHRONOUS             0x02

/* Modem Class          */
#define MODEM_CLASS_UNSPECIFIED         0x0000
#define MODEM_CLASS_V22                 0x0001
#define MODEM_CLASS_V22BIS              0x0002
#define MODEM_CLASS_V32                 0x0003
#define MODEM_CLASS_V32BIS              0x0004
#define MODEM_CLASS_PROPRIETARY         0x00FE


typedef struct GCCIComSF01VActSAPCnf
   {
      WORD              SFLength;       /* Variable                          */
      BYTE              SFIdentifier;   /* GCCI_COM_SFID_PRM_SPECIFIC        */
      BYTE              SFType;         /* GCCI_COM_SF_PCM_INFORMATION       */
      BYTE              NetworkType;    /* GCCI_COM_NET_TYPE_VSERIES         */
      BYTE              LocalNumCount;  /* Number of local Numbers           */
      WORD              BChannelsDef;   /* Number of defined B channels      */
      VCHANNELINFO      ChanInfoArr[1]; /* Array of channel info.s           */
      LOCALNUMINFO      LocalNumArr[1]; /* Array of Local Numbers            */
   } GCCI_COM_SF01V_ACT_SAP_CNF;

/* NetworkType */
#define GCCI_COM_NET_TYPE_VSERIES       0x02


/* Primitive Specific Subfield #2 for PCM_ACTIVATE_SAP Confirm */
typedef struct GCCIComSF02ActSAPCnf
   {
      WORD              SFLength;       /* sizeof(GCCI_COM_SF02_ACT_SAP_CNF)*/
      BYTE              SFIdentifier;   /* GCCI_COM_SFID_PRM_SPECIFIC       */
      BYTE              SFType;         /* GCCI_COM_SF_UPPER_DISPATCH       */
      WORD (CALL16PTR   SFCCReqEP)(VOID PTR16);   /* PCM Call Control Req Entry Pt    */
      WORD (CALL16PTR   SFCCRspEP)(VOID PTR16);   /* PCM Call Control Rsp Entry Pt    */
      WORD (CALL16PTR   SFRASReqEP)(VOID PTR16);  /* RAS Request Entry Pt address     */
      WORD (CALL16PTR   SFRASRspEP)(VOID PTR16);  /* RAS Response Entry Point address */
   } GCCI_COM_SF02_ACT_SAP_CNF;

/* Subfield Type       */
#define GCCI_COM_SF_UPPER_DISPATCH      0x02


/* Primitive Specific Subfield #3 (CNM Data) for PCM_ACTIVATE_SAP Confirm   */
typedef struct GCCIComSF03ActSAPCnf
   {
      WORD              SFLength;       /* sizeof(GCCI_COM_SF03_ACT_SAP_CNF)*/
                                        /* + (SizeofTracePts * NumTracePts) */
                                        /* + (SizeofDumps    * NumDumps   ) */
      BYTE              SFIdentifier;   /* GCCI_COM_SFID_PRM_SPECIFIC       */
      BYTE              SFType;         /* GCCI_COM_SF_CNM_DATA             */
      WORD              NumTracePts;    /* Number of Trace Points below     */
      WORD              SizeofTracePts; /* sizeof(GCCI_COM_TRACE_PT_DEF)    */
      WORD              NumDumps;       /* Number of Data Dumps  below      */
      WORD              SizeofDumps;    /* sizeof(GCCI_COM_DUMP_DEF)        */
      /* Add array of GCCI_COM_TRACE_PT_DEF Trace Point Definitions here.   */
      /* Add array of GCCI_COM_DUMP_DEF     Data Dump   Definitions here.   */
   } GCCI_COM_SF03_ACT_SAP_CNF;

/* Subfield Type       */
#define GCCI_COM_SF_CNM_DATA            0x03


/* CNM Data Subfield - Trace Point Definitions for PCM_ACTIVATE_SAP Confirm  */

#define GCCI_COM_MAX_TRACENAME_LEN 12

typedef struct GCCIComTracePtDef
   {
      WORD              TracePtID;      /* Trace Point ID assigned by caller */
      WORD              TraceType;      /* Trace Type (API/DATA/EVENT)       */
                                        /* ASCII Name for Trace Point        */
      BYTE              TraceName[GCCI_COM_MAX_TRACENAME_LEN];
   } GCCI_COM_TRACE_PT_DEF;

/* Trace Type          */
#define GCCI_CNM_TRACE_TYPE_API        0x0001
#define GCCI_CNM_TRACE_TYPE_DATA       0x0002
#define GCCI_CNM_TRACE_TYPE_EVENT      0x0003


/* CNM Data Subfield - Data Dump Definitions for PCM_ACTIVATE_SAP Confirm    */
typedef struct GCCIComDumpDef
   {
      WORD              DumpID;
      WORD              Reserved;
      BYTE              DumpName[12];
   } GCCI_COM_DUMP_DEF;


/*****************************************************************************/
/*                                                                           */
/*                 PCM_DEACTIVATE_SAP Request Primitive                      */
/*                                                                           */
/*****************************************************************************/

/* Control Information for PCM_DEACTIVATE_SAP Request */
typedef struct GCCIComCIDeactSAPReq
   {
      WORD              CtrlInfoLength; /* sizeof(GCCI_COM_CI_DEACT_SAP_REQ) */
      WORD              PrimitiveCode;  /* GCCI_COM_PC_DEACT_SAP_REQ         */
      WORD              DataInfoLength; /* 0                                 */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_P_SAP_ID         */
      DWORD             P_SAP_ID;       /* Provider assigned SAP id          */
      DWORD             Reserved1;      /* 0                                 */
      DWORD             SysLogID;       /* System Log identifier             */
   } GCCI_COM_CI_DEACT_SAP_REQ;

/* PrimitiveCode */
#define GCCI_COM_PC_DEACT_SAP_REQ       0x0C0E


/*****************************************************************************/
/*                                                                           */
/*                 PCM_DEACTIVATE_SAP Confirm Primitive                      */
/*                                                                           */
/*****************************************************************************/

/* Control Information for PCM_DEACTIVATE_SAP Confirm */
typedef struct GCCIComCIDeactSAPCnf
   {
      WORD              CtrlInfoLength; /* sizeof(GCCI_COM_CI_DEACT_SAP_CNF) */
      WORD              PrimitiveCode;  /* GCCI_COM_PC_DEACT_SAP_CNF         */
      WORD              DataInfoLength; /* 0                                 */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_SAP_ID         */
      DWORD             U_SAP_ID;       /* User assigned SAP id              */
      GCCI_COM_COMSTATUS CommonStatus;  /*                                   */
   } GCCI_COM_CI_DEACT_SAP_CNF;

/* PrimitiveCode */
#define GCCI_COM_PC_DEACT_SAP_CNF       0xCC0E


/*****************************************************************************/
/*                                                                           */
/*               PCM_DEACTIVATE_SAP Indication Primitive                     */
/*                                                                           */
/*****************************************************************************/

/* Control Information for PCM_DEACTIVATE_SAP Indication */
typedef struct GCCIComCIDeactSAPInd
   {
      WORD              CtrlInfoLength; /* sizeof(GCCI_COM_CI_DEACT_SAP_IND) */
      WORD              PrimitiveCode;  /* GCCI_COM_PC_DEACT_SAP_IND         */
      WORD              DataInfoLength; /* 0                                 */
      BYTE              Selector;       /* GCCI_COM_SEL_PCM                  */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_U_SAP_ID         */
      DWORD             U_SAP_ID;       /* User assigned SAP id              */
      GCCI_COM_COMSTATUS CommonStatus;  /*                                   */
   } GCCI_COM_CI_DEACT_SAP_IND;

/* Primitive Code for PCM_DEACTIVATE_SAP Indication */
#define GCCI_COM_PC_DEACT_SAP_IND       0x4C0E


/*****************************************************************************/
/*                                                                           */
/*                  Initialize PCM (from MER) Control Block                  */
/*                                                                           */
/*****************************************************************************/

/* Init_PCM Control Block */
typedef struct GCCIComMERInitPCM
   {
      WORD (CALL16PTR   MERRegEP)(VOID PTR16);    /* MER GET_CONFIG and REG_MODULE     */
                                        /*   Entry Point address             */
      WORD              GCCI_Version;   /* Version of GCCI API               */
      WORD              num_PCMNames;   /* Number of PCM Names configured    */
                                        /*   for this PCM EntryPoint in      */
                                        /*   NETWORK.INI.                    */
      BYTE        PTR16 PCMNames[10];   /* array of pointers to PCMName      */
                                        /*   ASCIIZ character strings        */
   } GCCI_COM_MER_INITPCM;


/*****************************************************************************/
/*                                                                           */
/*                  Config Data returned by GET_CONFIG command               */
/*                  Pointer returned is to typedef GCCI_keyword_data         */
/*                                                                           */
/*****************************************************************************/

typedef struct GCCI_keyword_parm        /* parameter info returned for each */
                                        /*    parameter of a keyword        */
   {
      WORD length;                      /* length of parameter string        */
      BYTE parm_string[1];              /* ASCIIZ string, null terminated    */
   } GCCI_KEYWORD_PARM;

typedef struct GCCI_keyword_data        /* Config data returned by the       */
                                        /*    GET_CONFIG command             */
   {
      struct GCCI_keyword_data PTR16 next_keyword;
      struct GCCI_keyword_data PTR16 prev_keyword;
      BYTE keyword[16];                 /* ASCIIZ keyword, truncated to      */
                                        /*    15 characters, alphabetic      */
                                        /*    characters in uppercase        */
      WORD num_parms;                   /* number of parms for this keyword  */
      GCCI_KEYWORD_PARM parm;           /* GCCI_keyword_parm is repeated for */
                                        /*    each parameter of the keyword  */
   } GCCI_KEYWORD_DATA;


/*****************************************************************************/
/*                                                                           */
/*                  GET_CONFIG Command                                       */
/*                                                                           */
/*****************************************************************************/

/* Control Information for GET_CONFIG Command */
typedef struct GCCIComCIGetConfig
   {
      WORD              CtrlInfoLength; /* sizeof(GCCI_COM_CI_GET_CONFIG)    */
      WORD              PrimitiveCode;  /* GCCI_COM_PC_GET_CONFIG            */
      WORD              DataInfoLength; /* 0                                 */
      BYTE              Selector;       /* 0                                 */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_NONE             */
      DWORD             Identifier;     /* 0                                 */
      BYTE              PCMName[16];    /* ASCIIZ PCM Name for config data   */
      GCCI_KEYWORD_DATA PTR16 PCMConfigData; /* Ptr to returned config data  */
   } GCCI_COM_CI_GET_CONFIG;

/* PrimitiveCode */
#define GCCI_COM_PC_GET_CONFIG          0x0F01


/*****************************************************************************/
/*                                                                           */
/*                  REG_MODULE Command                                       */
/*                                                                           */
/*****************************************************************************/

/* Control Information for REG_MODULE Command */
typedef struct GCCIComCIRegModule
   {
      WORD              CtrlInfoLength; /* sizeof(GCCI_COM_CI_REG_MODULE)    */
      WORD              PrimitiveCode;  /* GCCI_COM_PC_REG_MODULE            */
      WORD              DataInfoLength; /* 0                                 */
      BYTE              Selector;       /* 0                                 */
      BYTE              IDType;         /* GCCI_COM_ID_TYPE_NONE             */
      DWORD             Identifier;     /* 0                                 */
      BYTE              PCMName[16];    /* ASCIIZ PCM Name from Module Enbler*/
      WORD (CALL16PTR   PCMActSapEP)(); /* PCM Activate_SAP Entry Pt address */
      WORD              PCM_Version;    /* PCM Version Number                */
   } GCCI_COM_CI_REG_MODULE;

/* PrimitiveCode */
#define GCCI_COM_PC_REG_MODULE          0x0F02


/*****************************************************************************/
/* Common Primitive Control Information Defines                              */
/*                                                                           */
/*****************************************************************************/
/* Selector */
#define GCCI_COM_SEL_PCM                 0x20


/* Identifier Type */
#define GCCI_COM_ID_TYPE_NONE            0x00
#define GCCI_COM_ID_TYPE_U_CI_ID         0x01
#define GCCI_COM_ID_TYPE_U_CE_ID         0x01
#define GCCI_COM_ID_TYPE_U_DEV_ID        0x01
#define GCCI_COM_ID_TYPE_P_CI_ID         0x02
#define GCCI_COM_ID_TYPE_P_CE_ID         0x02
#define GCCI_COM_ID_TYPE_P_DEV_ID        0x02
#define GCCI_COM_ID_TYPE_U_SAP_ID        0x03
#define GCCI_COM_ID_TYPE_P_SAP_ID        0x04
