/*
*   Module name :     NETSAMPL
*
*   Function :	      Demo program for NETBIOS
*
*   Usage :	      NETSAMPL	 {SEND|RECV} [Adapter={0|1}]
*					     [Lclname=aaaaaaaaaaaaaaa]
*					     [Netname=bbbbbbbbbbbbbbb]
*
*				 where the two names can have up to
*				       fifteen characters.
*
*				 defaults: SEND
*					   Adapter=0
*					   Lclname=NETSAMPLSEND
*					   Netname=NETSAMPLRECV
*		      NETSAMPL	 ?
*
*   External modules: none
*
*   Return codes :    0    if successful
*		      1    if a parameter was bad and/or Help was given
*		      x    if an NCB fails the ncb.retcode is returned
*
*
*/

#include <stdio.h>
#include <ctype.h>

#include "ncb.h"
#include "ncbcmds.h"
#include "codetbl.h"
#include "neterror.h"

typedef unsigned char  BYTE;
typedef unsigned short WORD;
typedef char far *     ADDRESS;

/* global variables */

NCB  Ncb;			/* NCB used for all commands	      */
BYTE Mode;			/* 0 = Send, 1 = Recv		      */
BYTE Lana;			/* 0 or 1			      */
BYTE LclName[16];		/* local name to add to the network   */
BYTE NetName[16];		/* remote name to call/listen for     */
BYTE Message[80];		/* send/recv buffer		      */

#define  NET_LSN	 6	/* maximum number of local sessions   */
#define  NET_NUM	12	/* maximum number of outstanding NCBs */
#define  SEND		 0	/* Mode flag for SEND		      */
#define  RECV		 1	/* Mode flag for RECEIVE	      */


/**********************************************************************/
/*
**  main	Accepts the arguments and provides help if requested
**		RESETs the specified PC Network Adapter
**		ADDs the localname to the network
**		Initiates a CALL of the remotename over the network
**		When the CALL is answered, a message is SENT to the
**		  answering network station over the new session
**		HANGs UP the session with the remote station
**		RESETs the specified adapter
**
**		Error conditions:
**			First parameter not SEND or RECV
**			Incorrect Lana number specified
**			NAME already exists on the network
**			Fatal NETBIOS error 0x..
**
*/
main ( argc, argv )
int argc;
char *argv[];
{

	/*------------------------------------------------------------+
	|   Make sure NETBIOS is installed!			      |
	+------------------------------------------------------------*/

	if (Installed(0) < 2 && Installed(1) < 2) {
		printf( ">> Netbios and/or Adapter handler not installed!\n" );
		Terminate( " ", 0);
	}

	/*------------------------------------------------------------+
	|  Set the default parameters and initial values	      |
	+------------------------------------------------------------*/

	Mode = SEND;
	Lana = 0;
	memset( LclName, 0, 16);
	memset( NetName, 0, 16);

	if ( GrabArgs( argc, argv ) )  {
		Help();
		exit(1);
	}

	/*------------------------------------------------------------+
	|  If a name wasn't given, use the defaults                   |
	+------------------------------------------------------------*/

	if ( ! LclName[0] )
		strncpy(LclName,(Mode==SEND ?"NETSAMPLSEND":"NETSAMPLRECV"),15);
	if ( ! NetName[0] )
		strncpy(NetName,(Mode==SEND ?"NETSAMPLRECV":"NETSAMPLSEND"),15);

	/*------------------------------------------------------------+
	|  To make sure we start with a clean adapter, we reset it.   |
	+------------------------------------------------------------*/

	NetReset( Lana, NET_LSN, NET_NUM );
	if (Ncb.retcode)
		Terminate( "Reset", Ncb.retcode );

	/*------------------------------------------------------------+
	|  Add the name by which we wish to be known to the network   |
	+------------------------------------------------------------*/

	NetAddName( Lana, LclName );
	if (Ncb.retcode)
		Terminate( "AddName", Ncb.retcode );

	/*------------------------------------------------------------+
	|  If we are acting as the SENDing station, we next do:       |
	|     a CALL to the other station, and if it is answered,     |
	|     a SEND of our message to the other station.	      |
	|  Otherwise, we are the RECEIVing station and we next do:    |
	|     a LISTEN for a CALL from the other station, and when    |
	|	it occurs,					      |
	|     a RECEIVE to accept one message from the other station. |
	+------------------------------------------------------------*/

	if ( Mode == SEND )  {
		NetCall( Lana, LclName, NetName );
		if (Ncb.retcode)
			Terminate( "Call", Ncb.retcode );

		strcpy( Message, "Hello there!" );

		NetSend( Lana, Ncb.lsn, Message, strlen(Message) );
		if (Ncb.retcode)
			Terminate( "Send", Ncb.retcode );
	}
	else  {
		NetListen( Lana, LclName, NetName );
		if (Ncb.retcode)
			Terminate( "Listen", Ncb.retcode );

		NetReceive( Lana, Ncb.lsn, Message, sizeof Message );
		if (Ncb.retcode)
			Terminate( "Receive", Ncb.retcode );

		printf( "Received Message:\n" );
		printf( "  \"%s\"\n", Message);
	}

	/*------------------------------------------------------------+
	|  We terminate the session made above with the CALL/LISTEN   |
	|  by doing a HANGUP of the session.			      |
	+------------------------------------------------------------*/

	NetHangup( Lana, Ncb.lsn );
	if (Ncb.retcode && Ncb.retcode != 0x0A)
		Terminate( "Hangup", Ncb.retcode );

	/*------------------------------------------------------------+
	|  Finally, we reset the adapter to remove all traces of our  |
	|  ever using the adapter.				      |
	+------------------------------------------------------------*/

	NetReset( Lana, NET_LSN, NET_NUM );
	if (Ncb.retcode)
		Terminate( "Reset", Ncb.retcode );

	exit(0);
}


/**********************************************************************/
/*
** NetReset	Resets the NETBIOS interface status, clears the name
**		and session tables, and aborts all sessions.
**
**		Accepts the adapter number, max sessions, and max
**		pending NCBs.
**
**		Returns the NCB return code.
*/

NetReset( lana, lsn, num )
BYTE lana, lsn, num;
{
	printf( "Resetting lan adapter %u ... ", lana);

	memset( &Ncb, 0, sizeof(NCB) );
	Ncb.command  = NCB_RESET;
	Ncb.lana_num = lana;
	Ncb.lsn      = lsn;
	Ncb.num      = num;

	TRNcmd( &Ncb );

	if ( ! Ncb.retcode )
		puts( "Ok");

	return (Ncb.retcode);
}

/**********************************************************************/
/*
** NetAddName	Adds a 16-character name to the table of names.  The
**		name must be unique across the network.  This is a
**		name that this station will be known by.
**
**		Accepts the adapter number and the char array holding
**		the name.
**
**		Returns the NCB return code.
*/

NetAddName( lana, name )
BYTE lana;
BYTE *name;
{
	printf( "Adding the local name \"%s\" to the network ... ", name);

	memset( &Ncb, 0, sizeof(NCB) );
	Ncb.command  = NCB_ADDNAME;
	Ncb.lana_num = lana;
	strncpy( Ncb.name, name, 16 );

	TRNcmd( &Ncb );

	if ( ! Ncb.retcode )
		puts( "Ok");

	return (Ncb.retcode);
}

/**********************************************************************/
/*
** NetCall	Opens a session with another name specified in the
**		NCB.CALLNAME field using the local name specified in
**		the NCB.NAME field.
**
**		Accepts the adapter number and the char arrays holding
**		the local and remote names.
**
**		Returns the NCB return code.  If successful, the session
**		number is returned in the NCB.LSN field.
*/

#define RECV_TIMEOUT	10
#define SEND_TIMEOUT	10

NetCall( lana, lclname, rmtname)
BYTE lana;
BYTE *lclname, *rmtname;
{
	printf( "Calling the remote station \"%s\" ... ", rmtname);

	memset( &Ncb, 0, sizeof(NCB) );
	Ncb.command  = NCB_CALL;
	Ncb.lana_num = lana;
	Ncb.rto      = RECV_TIMEOUT<<1; 	/* times 2 since in   */
	Ncb.sto      = SEND_TIMEOUT<<1; 	/* steps of 500 msecs */
	strncpy( Ncb.name, lclname, 16 );
	strncpy( Ncb.callname, rmtname, 16 );

	TRNcmd( &Ncb );

	if ( ! Ncb.retcode )
		puts( "Ok");

	return (Ncb.retcode);
}

/**********************************************************************/
/*
** NetListen	Enables a session to be opened with the name specified
**		in the NCB.CALLNAME field, using the name specified in
**		the NCB.NAME field.
**
**		Accepts the adapter number and the char arrays holding
**		the local and remote names.
**
**		Returns the NCB return code.  If successful, the session
**		number is returned in the NCB.LSN field.
**
**		Note: A Listen command will NOT timeout, but it occupies
**		a session entry and is considered a pending session in the
**		information returned by a NCB.STATUS command.
*/

NetListen( lana, lclname, rmtname)
BYTE lana;
BYTE *lclname, *rmtname;
{
	printf( "Listening for a call from the remote station \"%s\" ... ", rmtname);

	memset( &Ncb, 0, sizeof(NCB) );
	Ncb.command  = NCB_LISTEN;
	Ncb.lana_num = lana;
	Ncb.rto      = RECV_TIMEOUT<<1; 	/* times 2 since in   */
	Ncb.sto      = SEND_TIMEOUT<<1; 	/* steps of 500 msecs */
	strncpy( Ncb.name, lclname, 16 );
	strncpy( Ncb.callname, rmtname, 16 );

	TRNcmd( &Ncb );

	if ( ! Ncb.retcode )
		puts( "Ok");

	return (Ncb.retcode);
}

/**********************************************************************/
/*
** NetSend	Sends data to the session partner as defined by the
**		session number in the NCB.LSN field.  The data to send
**		is in the buffer pointed to by the NCB.BUFFER field.
**
**		Accepts the adapter number, the session number,
**		the char array holding the message to be sent, and
**		the length of the message in that array.
**
**		Returns the NCB return code.
*/

NetSend( lana, lsn, message, length )
BYTE lana, lsn;
BYTE *message;
WORD length;
{
	printf( "Sending the following message to the answering station:\n" );
	printf( "  \"%s\" ... ", message);

	memset( &Ncb, 0, sizeof(NCB) );
	Ncb.command  = NCB_SEND;
	Ncb.lana_num = lana;
	Ncb.lsn      = lsn;
	Ncb.buffer   = message;
	Ncb.length   = length;

	TRNcmd( &Ncb );

	if ( ! Ncb.retcode )
		puts( "Ok");

	return (Ncb.retcode);
}

/**********************************************************************/
/*
** NetReceive	Receives data from the session partner that sends data
**		to this station.
**
**		Accepts the adapter number, the session number,
**		the char array to hold the message received, and
**		the maximum length the message may occupy in that
**		array.
**
**		Returns the NCB return code and, if successful,
**		the received data in the buffer.
*/

NetReceive( lana, lsn, buffer, length )
BYTE lana, lsn;
ADDRESS buffer;
WORD length;
{
	printf( "Receiving a message ..." );

	memset( &Ncb, 0, sizeof(NCB) );
	Ncb.command  = NCB_RECEIVE;
	Ncb.lana_num = lana;
	Ncb.lsn      = lsn;
	Ncb.buffer   = buffer;
	Ncb.length   = length-1;

	TRNcmd( &Ncb );

	if ( ! Ncb.retcode )
		puts( "Ok");

	buffer[length-1] = '\0';

	return (Ncb.retcode);
}

/**********************************************************************/
/*
** NetHangup	Closes the session with another name on the network
**		specified by the session number.
**
**		Accepts the adapter number and session number.
**
**		Returns the NCB return code.
*/

NetHangup( lana, lsn )
BYTE lana, lsn;
{
	printf( "Hanging up the session ... " );

	memset( &Ncb, 0, sizeof(NCB) );
	Ncb.command  = NCB_HANGUP;
	Ncb.lana_num = lana;
	Ncb.lsn      = lsn;

	TRNCmd( &Ncb );

	if ( ! Ncb.retcode || Ncb.retcode == 0x0A)
		puts( "Ok");

	return (Ncb.retcode);
}


/**********************************************************************/
/*
**  GrabArgs	Parses the command line arguments.  The first letters
**		of each possible parameter keyword are sufficient to
**		identify the parameter.
**
**		Accepts the standard argc,argv parameters
**
**		Modifies the global variables Mode, Lana, LclName, and
**		NetName if successful, or produces an error message and exits.
*/

GrabArgs( argc, argv )
int argc;
char *argv[];
{
	char *cp, *strchr();


	while( --argc )  {

		cp = *++argv;		/* ptr to next argument string */

		switch( toupper(*cp) )	{

			case 'S':       /* Send option */
					Mode = SEND;
					break;

			case 'R':       /* Receive option */
					Mode = RECV;
					break;

			case 'A':       /* Adapter number - must be followed
					   by an '=' and either 0 or 1  */

					cp = strchr( cp, '=' );
					if (! *cp)
						Terminate("Bad adapter number", 0);
					if (*++cp == '0')
						Lana = 0;
					else if (*cp == '1')
						Lana = 1;
					else
						Terminate("Bad adapter number", 0);
					break;

			case 'L':       /* Local network name - must be
					   followed by an '=' and up to
					   15 characters for the name  */

					cp = strchr( cp, '=' );
					if (! cp)
						Terminate("Bad local name", 0);
					strncpy( LclName, ++cp, 15 );
					break;

			case 'N':       /* Remote network name - must be
					   followed by an '=' and up to
					   15 characters for the name	*/

					cp = strchr( cp, '=' );
					if (! cp)
						Terminate("Bad remote name", 0);
					strncpy( NetName, ++cp, 15 );
					break;

			case '?':       /* Request for the help display */
					return 1;

			default:	printf( ">>Unrecognized argument: %s\n",
						cp );
					Terminate( " ", 0);
					break;
		}

	}

	return 0;
}

/**********************************************************************/
/*
**  Help	Displays directions for running this demo program
*/

Help()
{
	puts( " ");
	puts( " NETSAMPL   {Send|Recv} [Adapter={0|1}]          ");
	puts( "                        [Lclname=aaaaaaaaaaaaaaa]");
	puts( "                        [Netname=bbbbbbbbbbbbbbb]");
	puts( " ");
	puts( "            where the two names can have up to   ");
	puts( "                  fifteen characters.            ");
	puts( " ");
	puts( "            defaults: Adapter=0                  ");
	puts( "                      Lclname=NETSAMPL{SEND|RECV}");
	puts( "                      Netname=NETSAMPL{RECV|SEND}");
	puts( " ");

	return;
}

/**********************************************************************/
/*
**  Terminate	 Produce a formatted error message and
**		terminate the program
**
**		Accepts a character string holding the failed action
**		and an BYTE holding the return code.
**
**		DOES NOT RETURN
*/

#define BELL_CHAR	'\007'

Terminate( message, code )
BYTE *message;
BYTE code;
{
	extern char *TableLookUp();

	if (code)
		printf( "\n\n>> Error during %s: \n>>   %s%c\n",
			message, TableLookUp(NetErrorMsgs,code), BELL_CHAR );
	else	{
		printf( ">> %s\n", message);
		Help();
		code=1;
	}

	exit(code);
}

